/*
 * Name:    Txt282p.exe
 * Author:  Kouri (Kouri@hotbot.com)
 * Date:    April 18, 1999
 * Info:    Converts an ASCII text file into a TI-82 program file. Keeping
 *          notes (or cheat sheets) on your calc is now easier than ever.
 *          For example, let's say you have a ton of vocab words you have
 *          to memorize for a certain class. Type em up on your computer,
 *          run it through Txt282p, and send the file to your calc. Use
 *          TxtView, a simple assembly program I wrote, to view the file.
 *          See TxtView.asm for more information.
 * Usage:   Type "txt282p <filename> ["<description>"]" at the DOS prompt.
 *          <filename> is the name of the file that you wish to convert.
 *          <description> can be up to 16 characters and should be enclosed
 *          with quotes. It will be shown in TextView's Open File menu. If
 *          no description is given, <filename> will be used instead.
 * Credits: Thomas Hruska, for putting up with all my e-mail asking him for
 *          help and for actually taking the time to help me.
 *          Dan Eble, for releasing the source to string85.
 *          Dines Justesen, for also helping me out.
 *          Doug Torrence, for his Hacking the TI-82 columns.
 */

#include <stdio.h>

/* Function declarations */
void fputc_and_sum(int c);

/* Global variable declarations */
FILE *infile, *outfile;
unsigned long checksum=0x0B+0x06+0xFF+0xFF;

/* Start of main function */
int main(int argc, char *argv[]) {

/* Variable declarations */
 int c;
 unsigned long i;
 char filename[FILENAME_MAX];

 printf("\nText to 82p file converter - Kouri@hotbot.com\n");

 if (argc!=2 && argc!=3) {
  fprintf(stderr, "Usage: txt282p <filename> [\"<description>\"]\n");
  fprintf(stderr, "<filename> is the text file you wish to convert.\n");
  fprintf(stderr, "<description> can be up to 16 characters and should be enclosed with quotes.\n");
  fprintf(stderr, "If no description is provided, <filename> will be used instead.\n");
  return 1; }

 if ((infile = fopen(argv[1], "r")) == (FILE *) NULL) {
  fprintf(stderr, "Error: Can't read file: %s\n", argv[1]);
  return 2; }

 for (c=0; (argv[1][c] != '.') && (argv[1][c] != NULL); c++)
  filename[c]=argv[1][c];
 filename[c]  ='.';
 filename[c+1]='8';
 filename[c+2]='2';
 filename[c+3]='p';
 filename[c+4]=NULL;

 if ((outfile = fopen(filename, "wb")) == (FILE *) NULL) {
  fprintf(stderr, "Error: Can't write file: %s\n", filename);
  return 3; }

/* Calculate length of file */
 for (i=0; (i<16) && (argv[(argc==3)+1][i] != NULL); i++);
 for (i+=5; fgetc(infile) != EOF; i++);

/* Write header */
 fputs("**TI82**", outfile);
 fputc(0x1A, outfile);
 fputc(0x0A, outfile);
 fputc(0x00, outfile);
 fputs("82 text file generated by Txt282p         ", outfile);
 fputc((i+17)&0xFF, outfile);
 fputc((i+17)>>8, outfile);
 fputc(0x0B, outfile);
 fputc(0x00, outfile);
 fputc_and_sum((i+2)&0xFF);
 fputc_and_sum((i+2)>>8);
 fputc(0x06, outfile);

/* Write filename */
 for (c=0; (c<8) && (argv[1][c] != '.'); c++)
  fputc_and_sum(argv[1][c]);
 while (c++<8)
  fputc(0x00, outfile);

/* Write more data */
 fputc_and_sum((i+2)&0xFF);
 fputc_and_sum((i+2)>>8);
 fputc_and_sum(i&0xFF);
 fputc_and_sum(i>>8);

/* Write file ID used by TxtView */
 fputc(0xFF, outfile);
 fputc(0x00, outfile);

/* Write file description */
 for (c=0; (c<16) && (argv[(argc==3)+1][c] != NULL); c++)
  fputc_and_sum(argv[(argc==3)+1][c]);
 fputc(0x00, outfile);

/* Write actual text */
 rewind(infile);
 while ((c=fgetc(infile)) != EOF) {
  switch (c) {
   case '\n':
    fputc(0x00, outfile);
    break;
   case '\t':
    fputc_and_sum(0x06);
    break;
   case '[':
    fputc_and_sum(0xC1);
    break;
   default:
    fputc_and_sum(c);
    break;
  }; }

/* Write EOF used by TextView */
 fputc(0x00, outfile);
 fputc(0xFF, outfile);

/* Write checksum */
 fputc(checksum&0xFF, outfile);
 fputc(checksum>>8, outfile);

/* And we're done! */
 printf("File written: %s\n", filename, i);
 return 0; }

/* Function taken from string85 by Dan Eble - THANKS! */
void fputc_and_sum(c) {
 fputc(c, outfile);
 checksum+=c; }
